package fileMenu;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import java.io.IOException;

public class DialogDownload
extends JDialog
implements ActionListener, ListSelectionListener
{
   private JButton downloadButton = new JButton("Download");
   private JButton cancelButton = new JButton("Cancel");
   private ServerDirList fileList;
   private JTextField fileNameField = new JTextField();

   private ISavable savable;
   private String remoteFilename;
   private String serverDir;

   public DialogDownload(JFrame frame, ISavable savable0) {
      super(frame, true);
      savable = savable0;
      serverDir = savable.getServerDir();
      fileList = new ServerDirList(serverDir);

      JPanel myPanel = new JPanel(new BorderLayout());
      getContentPane().add(myPanel);

      myPanel.add(fileList);

      Box southPanel = Box.createVerticalBox();
      Box box = Box.createHorizontalBox();
      box.add(new JLabel("FileName:"));
      box.add(fileNameField);
      southPanel.add(box);

      box = Box.createHorizontalBox();
      box.add(Box.createHorizontalGlue());
      box.add(downloadButton);
      box.add(cancelButton);
      southPanel.add(box);
      
      myPanel.add(southPanel, BorderLayout.SOUTH);

      downloadButton.addActionListener(this);
      cancelButton.addActionListener(this);
      fileList.addListSelectionListener(this);

      pack();
      setLocationRelativeTo(frame);
   }

   public void showDialog(){
      String prefFilename = remoteFilename;
      remoteFilename = null;
      if( fileList.refreshList() ){
         fileList.setSelectedValue(prefFilename);
         fileNameField.setText(prefFilename);
         setVisible(true);
      }
   }

   public void actionPerformed(ActionEvent e) {
      if(cancelButton == e.getSource()) {
         setVisible(false);
      }
      else if(downloadButton == e.getSource()) {
         if( download()){
            remoteFilename = fileNameField.getText();
            setVisible(false);
         }
      }
   }

   public boolean wasSuccessful(){ return remoteFilename!=null; }
   public String getFileName(){ return remoteFilename; }
   public void setFileName(String s){ remoteFilename = s; }

   public void valueChanged(ListSelectionEvent e) {
      if( fileList.isReady() ){
         fileNameField.setText( fileList.getSelectedValue() );
      }
   }

   private boolean download(){
      String filename = fileNameField.getText();
      String err = ServerUtil.checkFileName(filename);
      if( err!=null ){
         showErr(err);
         return false;
      }
      // upload
      err = ServerUtil.submit(ServerUtil.READFILE, serverDir, new String[]{
            "filename", filename,
            });

      if( !err.startsWith("Ok\n") ){
         showErr(err);
         return false;
      }
      err = err.substring(3);


      try{
         savable.set(err);
      }catch(IOException e){
         showErr(e.getMessage());
         return false;
      }
      return true;
   }

   private void showErr(String err){
      JOptionPane.showMessageDialog(this, err, "Error", JOptionPane.ERROR_MESSAGE);
   }

}
