package fileMenu;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class DialogRename
extends JDialog
implements ActionListener, ListSelectionListener
{
   private JButton deleteButton = new JButton("Delete");
   private JButton renameButton = new JButton("Rename");
   private JButton cancelButton = new JButton("Finished");
   private ServerDirList fileList;
   private JTextField fileNameField = new JTextField();
   private JTextField newFileNameField = new JTextField();
   private JTextField passwordField = new JTextField();

   private String serverDir;

   public DialogRename(JFrame frame, String srvDir) {
      super(frame, true);
      serverDir = srvDir;
      fileList = new ServerDirList(serverDir);

      JPanel myPanel = new JPanel(new BorderLayout());
      getContentPane().add(myPanel);

      myPanel.add(fileList);

      Box southPanel = Box.createVerticalBox();
      Box box = Box.createHorizontalBox();
      box.add(new JLabel("FileName:"));
      box.add(fileNameField);
      southPanel.add(box);

      box = Box.createHorizontalBox();
      box.add(new JLabel("Password:"));
      box.add(passwordField);
      southPanel.add(box);

      box = Box.createHorizontalBox();
      box.add(new JLabel("New FileName:"));
      box.add(newFileNameField);
      southPanel.add(box);

      box = Box.createHorizontalBox();
      box.add(deleteButton);
      box.add(Box.createHorizontalGlue());
      box.add(renameButton);
      box.add(cancelButton);
      southPanel.add(box);

      myPanel.add(southPanel, BorderLayout.SOUTH);

      deleteButton.addActionListener(this);
      renameButton.addActionListener(this);
      cancelButton.addActionListener(this);
      fileList.addListSelectionListener(this);

      pack();
      setLocationRelativeTo(frame);
   }

   public void showDialog(){
      String prefFilename = null;
      if( fileList.refreshList() ){
         fileList.setSelectedValue(prefFilename);
         fileNameField.setText(prefFilename);
         setVisible(true);
      }
   }

   public void actionPerformed(ActionEvent e) {
      if(cancelButton == e.getSource()) {
         setVisible(false);
      }
      else if(deleteButton == e.getSource()) {
         if( delete() ){
            fileNameField.setText(null);
            newFileNameField.setText(null);
         }
      }
      else if(renameButton == e.getSource()) {
         if( rename()){
            fileNameField.setText(null);
            newFileNameField.setText(null);
         }
      }
   }

   public void valueChanged(ListSelectionEvent e) {
      if( fileList.isReady() ){
         fileNameField.setText( fileList.getSelectedValue() );
         newFileNameField.setText( fileList.getSelectedValue() );
      }
   }

   private boolean rename(){
      String filename = fileNameField.getText();
      String err = ServerUtil.checkFileName(filename);
      if( err!=null ){
         showErr(err);
         return false;
      }
      if( !fileList.containsEntry(filename) ){
         showErr("There is no file with that name on the server.");
         return false;
      }
      String newfilename = newFileNameField.getText();
      err = ServerUtil.checkFileName(newfilename);
      if( err!=null ){
         showErr(err);
         return false;
      }
      if( fileList.containsEntry(newfilename) ){
         showErr("There is already a file with that name on the server.");
         return false;
      }
      // rename
      err = ServerUtil.submit(ServerUtil.RENAMEFILE, serverDir, new String[]{
            "filename1", filename,
            "filename2", newfilename,
            "password", passwordField.getText()
            });

      if( !err.startsWith("Ok\n") ){
         showErr(err);
         return false;
      }
      fileList.removeEntry(filename);
      fileList.addEntry(newfilename);
      return true;
   }

   private boolean delete(){
      String filename = fileNameField.getText();
      String err = ServerUtil.checkFileName(filename);
      if( err!=null ){
         showErr(err);
         return false;
      }

      if( !fileList.containsEntry(filename) ){
         showErr("There is no file with that name on the server.");
         return false;
      }

      // delete
      err = ServerUtil.submit(ServerUtil.DELETEFILE, serverDir, new String[]{
            "filename", filename,
            "password", passwordField.getText(),
            });

      if( !err.startsWith("Ok\n") ){
         showErr(err);
         return false;
      }
      fileList.removeEntry(filename);
      return true;
   }

   private void showErr(String err){
      JOptionPane.showMessageDialog(this, err, "Error", JOptionPane.ERROR_MESSAGE);
   }
}
