package fileMenu;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class DialogUpload
extends JDialog
implements ActionListener, ListSelectionListener
{
   private JButton uploadButton = new JButton("Upload");
   private JButton cancelButton = new JButton("Cancel");
   private ServerDirList fileList;
   private JTextField fileNameField = new JTextField();
   private JTextField passwordField = new JTextField();

   private ISavable savable;
   private String remoteFileName;
   private String serverDir;

   public DialogUpload(JFrame frame, ISavable savable0) {
      super(frame, true);
      savable = savable0;
      serverDir = savable.getServerDir();
      fileList = new ServerDirList(serverDir);

      JPanel myPanel = new JPanel(new BorderLayout());
      getContentPane().add(myPanel);

      myPanel.add(fileList);

      Box southPanel = Box.createVerticalBox();
      Box box = Box.createHorizontalBox();
      box.add(new JLabel("FileName:"));
      box.add(fileNameField);
      southPanel.add(box);

      box = Box.createHorizontalBox();
      box.add(new JLabel("Password:"));
      box.add(passwordField);
      southPanel.add(box);

      box = Box.createHorizontalBox();
      box.add(Box.createHorizontalGlue());
      box.add(uploadButton);
      box.add(cancelButton);
      southPanel.add(box);

      myPanel.add(southPanel, BorderLayout.SOUTH);

      uploadButton.addActionListener(this);
      cancelButton.addActionListener(this);
      fileList.addListSelectionListener(this);

      pack();
      setLocationRelativeTo(frame);
   }

   public boolean wasSuccessful(){ return remoteFileName!=null; }
   public String getFileName(){ return remoteFileName; }
   public void setFileName(String s){ remoteFileName = s; }

   public void showDialog(){
      String prefFilename = remoteFileName;
      remoteFileName = null;
      if( fileList.refreshList() ){
         fileList.setSelectedValue(prefFilename);
         fileNameField.setText(prefFilename);
         setVisible(true);
      }
   }

   public void actionPerformed(ActionEvent e) {
      if(cancelButton == e.getSource()) {
         setVisible(false);
      }
      else if(uploadButton == e.getSource()) {
         if( upload()){
            remoteFileName = fileNameField.getText();
            setVisible(false);
         }
      }
   }

   public void valueChanged(ListSelectionEvent e) {
      if( fileList.isReady() ){
         fileNameField.setText( fileList.getSelectedValue() );
      }
   }

   private boolean upload(){
      String filename = fileNameField.getText();
      String err = ServerUtil.checkFileName(filename);
      if( err!=null ){
         showErr(err);
         return false;
      }
      // upload
      err = ServerUtil.submit(ServerUtil.WRITEFILE, serverDir, new String[]{
            "filename", filename,
            "password", passwordField.getText(),
            "contents", savable.get(false)
            });

      if( !err.startsWith("Ok\n") ){
         showErr(err);
         return false;
      }
      return true;
   }

   private void showErr(String err){
      JOptionPane.showMessageDialog(this, err, "Error", JOptionPane.ERROR_MESSAGE);
   }
}
