package fileMenu;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.filechooser.FileNameExtensionFilter;

public final class FileMenu
   extends JMenu
{
   JMenuItem saveMenuItem;
   ISavable savable;

   File localFile; // local file from which current object loaded
   String remoteFile; // remote file from which current object loaded

   private JFileChooser chooser;
   private DialogUpload dialogUpload;
   private DialogDownload dialogDownload;
   private DialogRename dialogRename;

   public FileMenu(boolean isApplet, ISavable savable0){
      super("File");
      savable = savable0;

      setMnemonic(KeyEvent.VK_F);
      getAccessibleContext().setAccessibleDescription("The file menu");

      addMenuListener(new MenuListener(){
         public void menuCanceled(MenuEvent arg0) {}
         public void menuDeselected(MenuEvent arg0) {}
         public void menuSelected(MenuEvent arg0) {
            if( saveMenuItem != null )
               saveMenuItem.setEnabled(localFile != null && savable.hasUnsavedChanges());
         }
      });

      JMenuItem menuItem = new JMenuItem("New", KeyEvent.VK_N);
      menuItem.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_N, ActionEvent.CTRL_MASK));
      menuItem.getAccessibleContext().setAccessibleDescription("New Graph");
      add(menuItem);
      menuItem.addActionListener(new ActionListener(){
         public void actionPerformed(ActionEvent arg0) { menuNew(); }
      });

      if( !isApplet ){
         menuItem = new JMenuItem("Open", KeyEvent.VK_O);
         menuItem.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_O, ActionEvent.CTRL_MASK));
         menuItem.getAccessibleContext().setAccessibleDescription("Open a file");
         add(menuItem);
         menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent arg0) { menuLoad(); }
         });

         menuItem = new JMenuItem("Save", KeyEvent.VK_S);
         menuItem.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_S, ActionEvent.CTRL_MASK));
         menuItem.getAccessibleContext().setAccessibleDescription("Save");
         add(menuItem);
         menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent arg0) { menuSave(); }
         });
         saveMenuItem = menuItem;

         menuItem = new JMenuItem("Save As", KeyEvent.VK_A);
         menuItem.getAccessibleContext().setAccessibleDescription("Save as a file");
         add(menuItem);
         menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent arg0) { menuSaveAs(); }
         });

         addSeparator();
      }

      menuItem = new JMenuItem("Download", KeyEvent.VK_D);
      menuItem.getAccessibleContext().setAccessibleDescription("Download from server");
      add(menuItem);
      menuItem.addActionListener(new ActionListener(){
         public void actionPerformed(ActionEvent arg0) { menuDownload(); }
      });

      menuItem = new JMenuItem("Upload", KeyEvent.VK_U);
      menuItem.getAccessibleContext().setAccessibleDescription("Upload to server");
      add(menuItem);
      menuItem.addActionListener(new ActionListener(){
         public void actionPerformed(ActionEvent arg0) { menuUpload(); }
      });

      menuItem = new JMenuItem("Rename/Delete", KeyEvent.VK_R);
      menuItem.getAccessibleContext().setAccessibleDescription("Rename or Delete server file");
      add(menuItem);
      menuItem.addActionListener(new ActionListener(){
         public void actionPerformed(ActionEvent arg0) { menuRename(); }
      });

      if( !isApplet ){
         addSeparator();

         menuItem = new JMenuItem("Exit", KeyEvent.VK_X);
         menuItem.getAccessibleContext().setAccessibleDescription("Exit");
         add(menuItem);
         menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent arg0) { menuExit(); }
         });
      }
   }

   private void setChooser(){
      chooser = new JFileChooser();
      FileNameExtensionFilter filter = new FileNameExtensionFilter(savable.getFileExtensionDesc(), savable.getFileExtension());
      chooser.setFileFilter(filter);
      chooser.setAcceptAllFileFilterUsed(false);
   }
   private File checkFileExtension( File f ){
      String fext = "."+savable.getFileExtension();
      String fn = f.getName();
      if( fn.endsWith(fext)) return f;
      return new File(f.getParent(), f.getName()+fext);
   }

   void menuNew(){
      if( processUnsavedData() ){
         savable.clear();
         savable.setNoUnsavedChanges();
         localFile = null;
         repaint();
      }
   }

   boolean menuSave(){
      if( localFile==null ) return menuSaveAs();
      if( ! savable.hasUnsavedChanges() )
         return true;
      return save(localFile);
   }
   boolean menuSaveAs(){
      if( chooser==null ) setChooser();
      File newfile;
      while(true){
         int returnVal = chooser.showSaveDialog(this);
         if(returnVal != JFileChooser.APPROVE_OPTION) return false;
         newfile = checkFileExtension(chooser.getSelectedFile());
         chooser.setSelectedFile(newfile);
         if( newfile==localFile || ! newfile.exists() ) break;
         returnVal = JOptionPane.showConfirmDialog(this, "Do you want to overwrite the existing file?", "Overwrite file", JOptionPane.YES_NO_CANCEL_OPTION);
         if( returnVal == JFileChooser.APPROVE_OPTION ) break;
         if( returnVal != JFileChooser.CANCEL_OPTION ) return false;
      }
      return save(newfile);
   }
   void menuLoad(){
      if( processUnsavedData() ){
         if( chooser==null ) setChooser();
         int returnVal = chooser.showOpenDialog(this);
         if(returnVal != JFileChooser.APPROVE_OPTION) return;
         File newfile = checkFileExtension(chooser.getSelectedFile());
         chooser.setSelectedFile(newfile);
         load(newfile);
      }
   }

   boolean menuUpload(){
      if( dialogUpload==null ){
         dialogUpload = new DialogUpload(null, savable);
      }
      dialogUpload.setFileName(remoteFile);
      dialogUpload.showDialog();
      if(! dialogUpload.wasSuccessful()) return false;
      remoteFile = dialogUpload.getFileName();
      if( localFile==null ){
         savable.setNoUnsavedChanges();
      }
      return true;
   }

   void menuDownload(){
      if( processUnsavedData() ){
         if( dialogDownload==null ){
            dialogDownload = new DialogDownload(null, savable);
         }
         dialogDownload.setFileName(remoteFile);
         dialogDownload.showDialog();
         if( !dialogDownload.wasSuccessful() ) return;

         localFile = null;
         remoteFile = dialogDownload.getFileName();
      }
   }

   void menuRename(){
      if( dialogRename==null ){
         dialogRename = new DialogRename(null, savable.getServerDir());
      }
      dialogRename.showDialog();
   }

   void menuExit(){
      if( processUnsavedData() ){
         System.exit(0);
      }
   }

   private boolean processUnsavedData(){
      if( savable.hasUnsavedChanges() ){
         boolean upload = saveMenuItem==null || ( localFile==null && remoteFile!=null );
         while(true){
            int r = JOptionPane.showConfirmDialog(this, "There are unsaved changes. Do you want to "+(upload ? "upload" :"save")+" them?", "Unsaved changes", JOptionPane.YES_NO_CANCEL_OPTION);
            if( r == JOptionPane.CANCEL_OPTION ) return false;
            else if( r == JOptionPane.YES_OPTION ){
               if( (!upload && menuSave()) || (upload && menuUpload()) ) break;
            }
            else break;
         }
      }
      return true;
   }
   private boolean save(File f){
      try{
         Writer writer = new FileWriter(f);
         writer.write(savable.get(true));
         writer.close();
         savable.setNoUnsavedChanges();
         localFile = f;
         return true;
      }catch(IOException fe){
         JOptionPane.showMessageDialog(this, fe.getMessage(), "File Error", JOptionPane.ERROR_MESSAGE);
         return false;
      }
   }
   private void load(File newfile){
      try{
         // read in whole file
         BufferedReader br = new BufferedReader( new FileReader(newfile) );
         StringBuilder s = new StringBuilder();
         String d = br.readLine();
         while( d!=null ){
            s.append(d).append('\n');
            d = br.readLine();
         }
         br.close();

         // parse it
         savable.set(s.toString());

         localFile = newfile;
      }catch(IOException fe){
         JOptionPane.showMessageDialog(this, fe.getMessage(), "File Error", JOptionPane.ERROR_MESSAGE);
      }
   }
}
