package fileMenu;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Arrays;
import java.util.Comparator;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionListener;

public class ServerDirList
   extends JPanel
   implements Comparator<String>
{
   private DefaultListModel listmodel = new DefaultListModel();
   private JList jlist = new JList(listmodel);
   private String serverDir;

   public ServerDirList(String serverDir0) {
      serverDir = serverDir0;
      setLayout( new BoxLayout(this, BoxLayout.Y_AXIS) );
      JButton refreshButton = new JButton("Refresh");

      Box box = Box.createHorizontalBox();
      box.add(new JLabel("Shared files:"));
      box.add(Box.createHorizontalGlue());
      box.add(refreshButton);

      add(box);
      add(new JScrollPane(jlist,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));

      refreshButton.addActionListener(new ActionListener(){
         public void actionPerformed(ActionEvent arg0) { refreshList(); }
      });
   }

   private boolean isReady = false;
   public boolean isReady(){ return isReady; }

   public boolean refreshList(){
      isReady = false;
      // upload
      String result = ServerUtil.submit(ServerUtil.READDIR, serverDir, null);
      if( result==null || ! result.startsWith("Ok\n") ){
         JOptionPane.showMessageDialog(this, "Error accessing server!", "Error", JOptionPane.ERROR_MESSAGE);
         listmodel.clear();
         return false;
      }
      // directory successfully read
      String[] filelist = (result.substring(3)).split("\n");
      Arrays.sort(filelist, this);
      listmodel.clear();
      listmodel.ensureCapacity(filelist.length);
      for(String s : filelist) listmodel.addElement(s);
      isReady = true;
      return true;
   }
   public void addListSelectionListener(ListSelectionListener lsl){
      jlist.addListSelectionListener(lsl);
   }
   public String getSelectedValue(){
      Object obj = jlist.getSelectedValue();
      return obj==null ? null : obj.toString();
   }
   public void setSelectedValue(String s){
      if(s!=null) jlist.setSelectedValue(s, true);
      else jlist.clearSelection();
   }
   public boolean containsEntry(String entry){
      if( !isReady ) return false;
      boolean found = false;
      for( int i = 0; !found && i<listmodel.getSize(); i++ ){
         found = entry.compareTo( listmodel.getElementAt(i).toString()) == 0;
      }
      return found;
   }
   public void removeEntry(String entry){
      if( !isReady ) return;
      listmodel.removeElement(entry);
   }
   public void addEntry(String entry){
      if( !isReady ) return;
      int i=0;
      while(i<listmodel.getSize() && compare(entry,(String)listmodel.get(i))>0 )
         i++;
      listmodel.add(i, entry);
   }


   public int compare(String strA, String strB) {
      return strA.compareToIgnoreCase(strB);
    }
}
