package lightsOutGraph.gui;
// Tile Puzzle Solver applet

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;

import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import lightsOutGraph.graphdata.Puzzle;
import fileMenu.ISavable;

public final class MainPanel
   extends JPanel
   implements ChangeListener, ISavable
{
   private int currentTab;
   /* tabs are:
    *  input: for textual import/export of puzzle
    *  board: for editing the board
    *  analyse: for analysing the game
    *  play: for playing the game
    */

   private JTabbedPane tabSet;

   private MyTab[] tabPanels = {
         new TabBoard(),
         new TabAnalyse(),
         new TabPlay(),
   };
   Puzzle puz=new Puzzle();

   public MainPanel() {
      setPreferredSize(new Dimension(600,500));

      tabSet = new JTabbedPane();

      // build row of tabs
      setLayout(new BorderLayout());
      add(tabSet);

      JPanel southPanel = new JPanel(new BorderLayout());
      JLabel copy = new JLabel(" Written by Jaap Scherphuis,  2004-2012");
      copy.setToolTipText(" email: puzzles@jaapsch.net   website: www.jaapsch.net/puzzles ");
      southPanel.add(copy,BorderLayout.EAST);
      southPanel.add(new JLabel("Version 1.4"),BorderLayout.WEST);
      add(southPanel,BorderLayout.SOUTH);

      for( int i=0; i<tabPanels.length; i++ ){
         MyTab tab = tabPanels[i];
         tabSet.addTab(tab.getLabel(),null,tab,tab.getToolTip());
      }
      currentTab=0;
      tabSet.setSelectedIndex(currentTab);
      tabSet.addChangeListener(this);
      tabPanels[currentTab].setPuzzle(puz);
   }

   public void stateChanged(ChangeEvent ce){
      tabPanels[currentTab].exit();
      int i=tabSet.getSelectedIndex();
      tabPanels[i].setPuzzle(puz);
      currentTab=i;
   }

   // ISavable
   // set/get used for loading/saving.
   public String get(boolean allowLarge) {
      return puz.toString();
   }
   public void set(String input) throws IOException{
      Reader reader = new StringReader(input);
      Puzzle p = Puzzle.parse(reader);
      reader.close();
      puz = p;
      tabPanels[currentTab].setPuzzle(puz);
      repaint();
   }
   // flag used to know if it needs to be saved
   public boolean hasUnsavedChanges(){
      return puz.hasUnsavedChanges();
   }
   // called after successful save, should clear the flag till savable contents change.
   public void setNoUnsavedChanges(){
      puz.setNoUnsavedChanges();
   }
   // clear all savable content. Called when the New menu option chosen.
   public void clear(){
      puz.getGraph().clear();
      puz.setNoUnsavedChanges();
      repaint();
   }

   // name of the server directory that this object is allowed to write to/read from. null means no server access allowed.
   public String getServerDir(){ return "lights"; }
   // Name of the file extension.
   public String getFileExtension(){ return "lo"; }
   // Description of the type of files with that extension.
   public String getFileExtensionDesc(){ return "Lights Out files"; }
}
